import numpy as np

def gauss_seidel(A, b, tol=1e-6, max_iter=1000):
    """
    Resuelve sistemas de ecuaciones lineales Ax = b usando el metodo de gauss-seidel

    Parametros:
        A: Matriz de coeficientes. b: Vector de terminos independientes.
        tol: Tolerancia para la convergencia. max_iter: Maximo numero de iteraciones.    
    Retorna:
        x: Solucion aproximada.
    """

    n = len(b)
    x = np.zeros(n)

    for k in range(max_iter):
        x_prev = x.copy()        
        for i in range(n):
            x[i] = (b[i] - sum(A[i,j] * x[j] for j in range(n) if j != i))/A[i,i]
        
        if np.linalg.norm(x - x_prev) < tol: #verificamos la convergencia de la solucion
            print(f"Solucion encontrada en {k + 1} iteraciones")
            return x
        
    print(f"No se logro la convergencia en {max_iter} iteraciones")
    return x




        


import numpy as np
from .fraccion import Fraccion


def lu_decomposition(coefficients, ind_terms):
    """Descompone una matriz en LU y resuelve Ax = b.

    Parametros:
        coefficients: Matriz de coeficientes (lista de listas).
        ind_terms: Vector de términos independientes (lista).

    Retorna:
        El vector solución (lista de objetos Fraccion).

    Raises:
        ValueError: Si no se puede realizar la descomposición LU.
    """
    n = len(coefficients)
    L = [[Fraccion(0)] * n for _ in range(n)]
    U = [[Fraccion(coefficients[i][j]) for j in range(n)] for i in range(n)]
    b = [Fraccion(val) for val in ind_terms]

    for i in range(n):
        L[i][i] = Fraccion(1)
        for j in range(i + 1, n):
            if U[i][i] == Fraccion(0):
                raise ValueError("No se puede realizar la descomposicion LU debido a un pivote cero.")
            factor = U[j][i] / U[i][i]
            L[j][i] = factor
            for k in range(i, n):
                U[j][k] = U[j][k] - factor * U[i][k]

    # Resolver Ly = b usando sustitucion hacia adelante
    y = [Fraccion(0)] * n
    for i in range(n):
        sum_ly = Fraccion(0)
        for j in range(i):
            sum_ly += L[i][j] * y[j]
        y[i] = (b[i] - sum_ly) / L[i][i]

    # Resolver Ux = y usando sustitucion hacia atras
    x = [Fraccion(0)] * n
    for i in range(n - 1, -1, -1):
        sum_ux = Fraccion(0)
        for j in range(i + 1, n):
            sum_ux += U[i][j] * x[j]
        x[i] = (y[i] - sum_ux) / U[i][i]

    return x




import numpy as np
import math as mt

def biseccion(func, a, b, tolerance=1e-7, max_iterations=100):
    """
    Encuentra una raíz de la función 'func' en el intervalo [a, b] usando el método de bisección.

    Parametros:       
        tolerance (float, opcional): La tolerancia para la raíz encontrada. Defaults to 1e-7.
        max_iterations (int, opcional): El número máximo de iteraciones. Defaults to 100.

    Retorna:
        Una aproximación de la raíz si se encuentra dentro de la tolerancia
        y el número máximo de iteraciones, de lo contrario devuelve None.
    """
    if np.sign(func(a)) == np.sign(func(b)):
        raise ValueError("La función debe tener signos opuestos en los extremos del intervalo.")
    
    itr = 0
    for i in range(max_iterations):
        c = (a + b) / 2
        itr = i + 1
        if np.abs(func(c)) < tolerance:
            return c, itr
        elif np.sign(func(c)) == np.sign(func(a)):
            a = c
        else:
            b = c        

    print(f"Advertencia: Número máximo de iteraciones alcanzado sin la tolerancia deseada. Última aproximación: {c}")
    return c, itr


import numpy as np
from .fraccion import Fraccion
def gauss_jordan(coefficients, ind_terms, imprimir_it=False):
    """ Resuelve un sistema lineal Ax = ind_terms por Gauss-Jordan.

        Parametros:
            coefficients: Matriz de coeficientes (lista de listas).
            ind_terms: Vector de términos independientes (lista).
            imprimir_it: valor boolean que determina si se imprimen las interaciones.
        Retorna:
            El vector solución (array) o None si no hay solución única.
    """
    n = len(ind_terms)
    
    equations_ls = [[Fraccion(coefficients[i][j]) for j in range(n)] + [Fraccion(ind_terms[i])] for i in range(n)]
    equations = np.array(equations_ls)
    for i in range(n):
        if i == 0 and imprimir_it : print("matriz inicial", *equations, sep= "\n")   
        if equations[i, i] == 0:
            for k in range(i + 1, n):
                if equations[k, i] != 0:
                    equations[[i, k]] = equations[[k, i]]
                    break
            else:
                return None       
                  
        pivote = equations[i, i]
        equations[i] = np.array([equations[i, l] / pivote for l in range(n + 1)])
        
        for j in range(n):
            if i != j:                
                factor = equations[j, i]
                equations[j] = np.array([equations[j, l] - factor * equations[i, l] for l in range(n + 1)])
        
        if imprimir_it : print('Iteracion: ' + str(i+1), *equations, sep= "\n")                       
    return equations[:, -1]






from .Biseccion import biseccion
from .Crammer import crammer
from .Descomposicion_LU import lu_decomposition
from .Gauss_Eliminacion import gauss_elimination
from .Gauss_Jordan import gauss_jordan
from .Gauss_Seidel import gauss_seidel
from .Jacobi import jacobi
from .fraccion import Fraccion


from AA20029UNO import crammer

# Ejemplo de uso
coefficients = [[3, -1, 4], [1, 2, -1], [2, -1, 1]]
ind_terms = [10, 3, 5]

solucion = crammer(coefficients, ind_terms)
print("Solución del sistema de ecuaciones por la regla de Cramer:")
for variable, valor in solucion.items():
    print(f"{variable} = {valor}")


import numpy as np
class Fraccion:
    def __init__(self, numerador, denominador=1):
        if isinstance(numerador, Fraccion):
            self.n = numerador.n
            self.d = numerador.d * denominador
        else:
            self.n = numerador
            self.d = numerador.denominator * denominador.numerator
        if self.d == 0:
            raise ValueError("El denominador no puede ser cero")
        if self.d < 0:
            self.n = -self.n
            self.d = -self.d
        self.simplificar()    
        
        """if denominador == 0:
            raise ValueError("El denominador no puede ser cero")
        
        if self.d < 0:
            self.n = -self.n
            self.d = -self.d
        self.simplificar()"""
    
    def simplificar(self):
        gcd = np.gcd(self.n, self.d)
        self.n //= gcd
        self.d //= gcd
    
    def __add__(self, otro):
        if isinstance(otro, int):
            otro = Fraccion(otro)
        return Fraccion(self.n * otro.d + otro.n * self.d, self.d * otro.d)
    
    def __sub__(self, otro):
        return Fraccion(self.n * otro.d - otro.n * self.d, self.d * otro.d)
    
    def __mul__(self, otro):
        if isinstance(otro, int):
            otro = Fraccion(otro)
        return Fraccion(self.n * otro.n, self.d * otro.d)
    
    def __truediv__(self, otro):
        if isinstance(otro, int):
            otro = Fraccion(otro)
        return Fraccion(self.n * otro.d, self.d * otro.n)
    
    def __neg__(self):
        return Fraccion(-self.n, self.d)
    
    def __repr__(self):
        return f"{self.n}/{self.d}" if self.d != 1 else str(self.n)


import numpy as np
def gauss_elimination(coefficients, ind_terms):

    """
        Resuelve un sistema de ecuaciones lineales por el metodo de eliminacion de guass

        Parametros:
            coefficients: Matriz de coeficientes (lista de listas o array).
            ind_terms: Vector de términos independientes (lista o array).

        Retorna:
            El vector solución (array) o None si no hay solución única.
    """

    coefficients = np.array(coefficients, dtype=float)
    ind_terms = np.array(ind_terms, dtype=float)
    n = len(ind_terms)
    equations = np.hstack([coefficients, ind_terms.reshape(-1,1)])
    for i in range(n):
        max_row = max(range(i, n), key=lambda r: abs(equations[r,i]))
        if equations[max_row, i] == 0:
            return None
        equations[[i, max_row]] = equations[[max_row, i]]

        for j in range(i + 1, n):
            factor = equations[j, i] / equations[i, i]
            equations[j,i:] -= factor * equations[i, i:]

    x = np.zeros(n)

    for i in range(n - 1, -1, -1):
        x[i] = (equations[i, -1] - np.dot(equations[i, i + 1:n], x[i + 1:n])) / equations[i,i]
    
    return x




import pathlib
from setuptools import find_packages, setup

HERE = pathlib.Path(__file__).parent

VERSION = '0.1.4' #Muy importante, deberéis ir cambiando la versión de vuestra librería según incluyáis nuevas funcionalidades
PACKAGE_NAME = 'AA20029UNO' #Debe coincidir con el nombre de la carpeta 
AUTHOR = 'Moises Aaron Alvarenga Acosta' #Modificar con vuestros datos
AUTHOR_EMAIL = 'moisesacosta8@gmail.com' #Modificar con vuestros datos
URL = 'https://github.com/Moises211' #Modificar con vuestros datos

LICENSE = 'MIT' #Tipo de licencia
DESCRIPTION = 'Una librería para resolver sistemas de ecuaciones lineales y no lineales.' #Descripción corta
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf-8') #Referencia al documento README con una descripción más elaborada
LONG_DESC_TYPE = "text/markdown"

#'numpy>=1.9.3'
#Paquetes necesarios para que funcione la libreía. Se instalarán a la vez si no lo tuvieras ya instalado
INSTALL_REQUIRES = [      
      'numpy>=1.10'
      ]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    install_requires=INSTALL_REQUIRES,
    license=LICENSE,
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Operating System :: OS Independent',
    ]
)


import numpy as np
from fractions import Fraction as StandardFraction
from .fraccion import Fraccion

def crammer(coefficients, ind_terms):
    """Resuelve un sistema 3x3 por la regla de Cramer.

        Parametros:
            coefficients: Matriz 3x3 de coeficientes (lista de listas).
            ind_terms: Vector de términos independientes (lista de 3 elementos).

        Retorna:
            Un diccionario con las soluciones {'x': valor, 'y': valor, 'z': valor}
            o un mensaje si no hay solución única.
    """

    def determinante(matriz_det):
        dimension = len(matriz_det)
        if dimension == 1:
            return matriz_det[0][0]
        elif dimension == 2:
            return (matriz_det[0][0] * matriz_det[1][1]) - (matriz_det[0][1] * matriz_det[1][0])
        else:
            determinante_val = Fraccion(0)
            for j in range(dimension):
                elemento = matriz_det[0][j]
                submatriz = [fila[:j] + fila[j+1:] for fila in matriz_det[1:]]
                determinante_submatriz = determinante(submatriz)
                signo = Fraccion(1) if j % 2 == 0 else Fraccion(-1)
                determinante_val += elemento * signo * determinante_submatriz
            return determinante_val

    # Convertir los coeficientes y términos independientes a objetos Fraccion
    matriz_coeficientes = [[Fraccion(StandardFraction(num)) for num in row] for row in coefficients]
    vector_terminos_independientes = [Fraccion(StandardFraction(num)) for num in ind_terms]

    det_A = determinante(matriz_coeficientes)

    if det_A == Fraccion(0):
        return "El sistema no tiene solución única (el determinante de la matriz de coeficientes es cero)."

    n = len(vector_terminos_independientes)
    soluciones = {}

    for i in range(n):
        matriz_modificada = [list(fila) for fila in matriz_coeficientes]
        for j in range(n):
            matriz_modificada[j][i] = vector_terminos_independientes[j]

        det_Ai = determinante(matriz_modificada)
        variable_name = ['x', 'y', 'z'][i]
        soluciones[variable_name] = det_Ai / det_A

    return soluciones

